/*
Sjisutf16 C++ Version 1.01
Converting the text from Shift_JIS to UTF-16
Copyright 2020 Sogaya

Borland C++ 5.5.1 for Win32 (BCC55)
Digital Mars Compiler Version 8.42n (DMC 8.57 [dm857c.zip] ), STLport 4.5.3
Embarcadero C++ 7.20 for Win32 (BCC101)
Embarcadero C++ 7.30 for Win32 (BCC102)
GCC 3.4.2 (mingw-special [Bloodshed Dev-C++ 4.9.9.2] )

*/

#include "ConvertWide.h"
#include "ErrorText.h"
#include "FinalEffect.h"
#include "Guide.h"
#include "HyphenTerm.h"
#include "ReadWrite.h"
#include "SearchBinary.h"
#include "StandardIO.h"

namespace {
    char C_SETTING_TEXT[] = "C:\\Sjisutf8\\set_sjisutf16.txt";
    char DATA_TEXT[] = "data.txt";
    char SETTING_TEXT[] = "set_sjisutf16.txt";
    char TEMPORARY_BINARY[] = "binary.tmp";
    char TEMPORARY_TEXT[] = "text.tmp";
    const bool DO_NOT_RECORD_PATH_NAME = false;
    const bool DO_NOT_REMOVE_ANCIENT_FILE = true;
    const bool DO_NOT_REMOVE_TEMPORARY_BINARY = true;
    const bool REMOVE_ANCIENT_FILE = false;
    const bool SEARCH_TABLE_IN_EACH_DIRECTORY = true;
    const bool SELECTED_DIRECTORY_DOES_NOT_HAVE_PRIORITY = false;
    const bool VERSION_WITH_COMMENTS = true;
    const bool VERSION_WITHOUT_COMMENTS = false;
    const char TAB_C = 9; // 09h
    const int BUFFER_ERROR = -1;
    const int UPPER_LIMIT_TO_FIELD_NUMBER = 3071; // F_SIZE - 1
    const std::string C_ERR = "err";
    const std::string TEXT_FILE = ".txt";
    double VERSION_NUMBER = 1.01;
    std::string PATH_NAME = "C:\\Sjisutf8\\MS932~UTF16.bin";
}

int main(int argc, char *argv[]) {
    using namespace std;

    bool SETTING_TEXT_IS_NOT_ACCESSIBLE;
    try {
        ifstream testFile(C_SETTING_TEXT);
        testFile.exceptions(ios_base::failbit);
        SETTING_TEXT_IS_NOT_ACCESSIBLE = false;
    } catch(...) {
        SETTING_TEXT_IS_NOT_ACCESSIBLE = true;
    }

    if( SETTING_TEXT_IS_NOT_ACCESSIBLE ) {
        try {
            ifstream testFile(SETTING_TEXT);
            testFile.exceptions(ios_base::failbit);
            SETTING_TEXT_IS_NOT_ACCESSIBLE = false;
        } catch(...) {
// Providing SETTING_TEXT in the current directory
            ofstream newFile(SETTING_TEXT);
            newFile << PATH_NAME << endl;
            newFile << "-0" << TAB_C << "CRLF" << endl;
            newFile << "-%" << TAB_C << "BOM" << endl;
        }

    }

    if( argc == 1 ) {
// Producing TEMPORARY_TEXT from standard input 
        StandardInput *i = new StandardInput();
        i->produceTemporaryTextFile();

// Converting TEMPORARY_TEXT from Shift_JIS to UTF-16 
        SearchTable *e = new SearchTable(PATH_NAME, SEARCH_TABLE_IN_EACH_DIRECTORY, SELECTED_DIRECTORY_DOES_NOT_HAVE_PRIORITY, DO_NOT_RECORD_PATH_NAME);
        i->setting(e);
        i->standard_input();

        remove(TEMPORARY_TEXT);
        delete e; delete i;
        return 0;
    }

// command parameters 
    bool carriage_return;
    int file_name_length, new_name_length;
    string file_name, new_name;
    ParameterList *listing = new ParameterList(argc, argv);
    HyphenTerms *h = new HyphenTerms(listing);
    h->setting();

    if( h->guideFile() ) {
        Guiding *g = new Guiding();
        g->versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
        delete g; delete h;
        exit(261);
    } else if( h->versionNumber() ) {
        Guiding *g = new Guiding();
        g->versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
        delete g; delete h;
        exit(262);
    }

    new_name = h->listName(1);
    new_name_length = new_name.length();
    if( new_name_length == 0 ) {
        new_name = h->newName();
    }

    if( h->lineFeed() ) {
        if( h->carriageReturn() ) {
            carriage_return = true;
        } else {
            carriage_return = false;
        }

    } else {
        carriage_return = true;
    }

    h->joints(); // joint_byte 
    h->labels(); // label_byte
    h->quotes(); // quote_byte 
    h->separators(); // separator_byte 

    FinalEffects *f = new FinalEffects( h->fileName(), new_name, h->pathName(), 
    h->atPoint(), 
    h->jointByte(), 
    carriage_return, h->checkLength(), h->countLines(), h->identicalTime(), h->noRemoving(), h->noSearching(), h->remove0Dh(), h->removeControllers(), h->priority(), h->produceTemporaryBinary(), h->whichTable(), h->withBOM() );

    if( f->whichTable() ) {
        try {
            ifstream dataFile(DATA_TEXT);
            dataFile.exceptions(ios_base::failbit);
            remove(DATA_TEXT);
        } catch(...) {
            ;
        }

    }

    new_name = f->newName();
    new_name_length = new_name.length();
    if( new_name_length == 0 ) {
        Guiding *g = new Guiding();
        g->versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
        delete g; delete f; delete h;
        exit(263);
    }

    file_name = f->fileName();
    file_name_length = file_name.length();
    if( file_name_length == 0 ) {
        if( f->produceTemporaryBinary() ) {
            ReadingWriting *r = new ReadingWriting(f, TEMPORARY_BINARY);

            if( f->removeControllers() ) {
                r->standard_input_skipping_to_produce_UTF16_CRLF();
            } else {
                r->standard_input_reading_to_produce_UTF16_CRLF();
            }

            r->renameBinaryFile( new_name, f->noRemoving() );
            delete r; delete f; delete h;
            return 0;
        }

        if( f->checkLength() ) {
            ReadingWriting *r = new ReadingWriting(f, TEMPORARY_BINARY);

            if( f->removeControllers() ) {
                r->standard_input_checking_skipping_to_produce_UTF16_CRLF();
            } else {
                r->standard_input_checking_to_produce_UTF16_CRLF();
            }

            r->renameBinaryFile( new_name, f->noRemoving() );
            delete r; delete f; delete h;
            return 0;
        }

// Producing TEMPORARY_TEXT from standard input 
        StandardInput *i = new StandardInput();
        i->produceTemporaryTextFile();

        i->setting(f);

        if( f->whichTable() ) {
            i->displayDataText();
        }

        if( f->removeControllers() ) {
            i->standard_input_of_non_controllers();
        } else {
            i->standard_input();
        }

        if(! f->noRemoving() ) {
            remove(TEMPORARY_TEXT);
        }

        delete i; delete f; delete h;
        return 0;
    }

    if( f->produceTemporaryBinary() ) {
        ReadingWriting *r = new ReadingWriting(f, TEMPORARY_BINARY);
        if( r->setting() ) {
            if( f->carriageReturn() ) {
                if( f->removeControllers() ) {
                    r->skipping_control_to_produce_UTF16_CRLF();
                } else {
                    r->reading_to_produce_UTF16_CRLF();
                }

            } else {
                if( f->removeControllers() ) {
                    r->skipping_control_to_produce_UTF16_LF();
                } else {
                    r->reading_to_produce_UTF16_LF();
                }

            }

            r->renameBinaryFile( new_name, f->noRemoving() );
        } else {
            exit(264);
        }

        delete r; delete f; delete h;
        return 0;
    }

    if( f->checkLength() ) {
        ReadingWriting *r = new ReadingWriting(f, TEMPORARY_BINARY);
        if( r->setting() ) {
            if( f->carriageReturn() ) {
                if( f->removeControllers() ) {
                    r->checking_skipping_control_to_produce_UTF16_CRLF();
                } else {
                    r->checking_length_to_produce_UTF16_CRLF();
                }

            } else {
                if( f->removeControllers() ) {
                    r->checking_skipping_control_to_produce_UTF16_LF();
                } else {
                    r->checking_length_to_produce_UTF16_LF();
                }

            }

            r->renameBinaryFile( new_name, f->noRemoving() );
        } else {
            exit(265);
        }

        delete r; delete f; delete h;
        return 0;
    }

    if( f->countLines() ) {
        ReadingWriting *r = new ReadingWriting(f, TEMPORARY_BINARY);
        if( r->setting() ) {
            int count = r->countLines(file_name);

            if( count == 1 ) {
                cout << count << " line" << endl;
            } else {
                cout << count << " lines" << endl;
            }

        } else {
            exit(266);
        }

        delete r; delete f; delete h;
        return 0;
    }

    ConvertWiden *c = new ConvertWiden();
    c->setting(f, file_name);
    c->ifErrorOccurs(BUFFER_ERROR);

    if( f->identicalTime() ) {
        c->produceBinaryFile(file_name, REMOVE_ANCIENT_FILE);
        c->renameBinaryFile( file_name, f->noRemoving() );
    }

    if( f->carriageReturn() ) {
        if( f->removeControllers() ) {
            c->skipping_control_to_produce_UTF16_CRLF(new_name);
        } else {
            c->reading_to_produce_UTF16_CRLF(new_name);
        }

    } else {
        if( f->remove0Dh() ) {
            if( f->removeControllers() ) {
                c->skipping_control_to_produce_UTF16_LF(new_name);
            } else {
                c->reading_to_produce_UTF16_LF(new_name);
            }

        } else {
            if( f->removeControllers() ) {
                c->skipping_control_to_produce_UTF16_text(new_name);
            } else {
                c->reading_to_produce_UTF16_text(new_name);
            }

        }

    }

    if( f->whichTable() ) {
        c->displayDataText();
    }

    if( SETTING_TEXT_IS_NOT_ACCESSIBLE ) {
        if(! f->noRemoving() ) {
            remove(SETTING_TEXT);
        }

    }

    delete c; delete f; delete h; delete listing;
    return 0;
}
