#include "SparseUTF16.h"

namespace {
    char C_MS932_UTF16_BIN[] = "C:\\Sjisutf8\\MS932~UTF16.bin";
    char C_SETTING_TEXT[] = "C:\\Sjisutf8\\set_sjisutf16.txt";
    char DATA_TEXT[] = "data.txt";
    char MS932_UTF16_BIN[] = "MS932~UTF16.bin";
    char SETTING_TEXT[] = "set_sjisutf16.txt";
    const bool PUT_ERROR_TEXTS = true;
    const bool PUT_NEW_LINE = true;
    const int FIRST_KANJI = 159;
    const int UTF16_KANJI_SIZE = 2;
    const int UTF16_KANJI_SIZE_1 = 3; // UTF16_KANJI_SIZE + 1 
    const int SEARCH_COUNT = 4;
    const std::string C_ERR = "err";
    const std::string EMPTY = "";
    const std::string TEXT_FILE = ".txt";
    const wchar_t ZERO_CHAR = 0;
}

std::string binary_file_in_current_directory() {
    using namespace std;

    int error_number;
    string s(MS932_UTF16_BIN);
    stringstream os;

    try {
        ifstream testFile(MS932_UTF16_BIN, ios::binary);
        testFile.exceptions(ios_base::failbit);
    } catch(...) {
        error_number = 286;
        os << "binary_file_in_current_directory() error-" << error_number << ": " << MS932_UTF16_BIN;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        s = EMPTY;
    }

    return s;
}

std::string binary_file_in_installation_directory() {
    using namespace std;

    int error_number;
    string s(C_MS932_UTF16_BIN);
    stringstream os;

    try {
        ifstream testFile(C_MS932_UTF16_BIN, ios::binary);
        testFile.exceptions(ios_base::failbit);
    } catch(...) {
        error_number = 287;
        os << "binary_file_in_installation_directory() error-" << error_number << ": " << C_MS932_UTF16_BIN;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        s = EMPTY;
    }

    return s;
}

std::string binary_file_in_path_directory() {
// directory included in Path 
    CommandPath p(PUT_ERROR_TEXTS);
    std::string g(MS932_UTF16_BIN);
    return p.pathName(g);
}

std::string binary_file_in_selected_directory() {
// directory that may have priority 
    using namespace std;

    bool SETTING_TEXT_IS_NOT_ACCESSIBLE;
    char c[C_SIZE];
    char name[C_SIZE];
    int error_number;
    string s;
    stringstream os;

    try {
        ifstream testFile(C_SETTING_TEXT, ios::binary);
        testFile.exceptions(ios_base::failbit);
        strcpy(name, C_SETTING_TEXT);
        SETTING_TEXT_IS_NOT_ACCESSIBLE = false;
    } catch(...) {
        SETTING_TEXT_IS_NOT_ACCESSIBLE = true;
    }

    if( SETTING_TEXT_IS_NOT_ACCESSIBLE ) {
        try {
            ifstream testFile(SETTING_TEXT, ios::binary);
            testFile.exceptions(ios_base::failbit);
            strcpy(name, SETTING_TEXT);
        } catch(...) {
            error_number = 288;
            os << "binary_file_in_selected_directory() error-" << error_number << ": " << SETTING_TEXT;

            ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
            x.setting();
            s = EMPTY;
            exit(error_number);
        }

    }

    ifstream settingFile(name);
    if( settingFile.getline(c, C_SIZE) ) { // Line-1
        string path_name(c);
        s = path_name;
    }

    if( s.length() > 0 ) {
        try {
            ifstream testFile( s.c_str(), ios::binary );
            testFile.exceptions(ios_base::failbit);
        } catch(...) {
            error_number = 289;
            os << "binary_file_in_selected_directory() error-" << error_number << ": " << s;

            ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
            x.setting();
            s = EMPTY;
        }

    }

    return s;
}

SparsePopulationUTF16::~SparsePopulationUTF16() {
    delete[] z;
}

SparsePopulationUTF16::SparsePopulationUTF16() {
    z = new char[Z_SIZE];
}

bool SparsePopulationUTF16::setting(std::string BINARY_TABLE) {
    using namespace std;

    bool r;
    int b, block_has_been_read, error_number, k;
    int count = 0;
    stringstream os;
    unsigned char u;
    wchar_t w = ZERO_CHAR;

// Reaeding BINARY_TABLE to produce a table in the main memory 
    try {
        ifstream testFile( BINARY_TABLE.c_str() );
        testFile.exceptions(ios_base::failbit);
        r = true;
    } catch(...) {
        r = false;
        error_number = 290;
        os << "setting(std::string & BINARY_TABLE) error-" << error_number << ": " << BINARY_TABLE;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    clear();
    ifstream tableFile( BINARY_TABLE.c_str(), ios::binary );
    for(;;) {
        tableFile.read(a, E_SIZE);
        block_has_been_read = tableFile.gcount();
        if( block_has_been_read == 0 ) {
            break;
        }

        count += 1;
        if( count < FIRST_KANJI ) {
// ASCII 
            u = (unsigned char)a[0];
            k = (int)u;
        } else {
// kanji 
            u = (unsigned char)a[0];
            b = (int)u;
            k = b << 8;

            u = (unsigned char)a[1];
            b = (int)u;
            k += b;
        }

        populated(k, 0, a[2]);
        populated(k, 1, a[3]);
    }

// Producing another table in the main memory 
    for(int n = 0; n < U_SIZE; ++n) {
        UTF16_CHAR[n] = w;
        ++w;
    }

    return r;
}

bool SparsePopulationUTF16::setting(SearchTable *e) {
    using namespace std;

    bool r;
    ErrorTexts *x;
    int b, beginning, block_has_been_read, error_number, k;
    int count = 0;
    string BINARY_TABLE;
    stringstream os;
    unsigned char u;
    wchar_t w = ZERO_CHAR;

// Finding MS932_UTF16_BIN 
    open_close[1] = binary_file_in_installation_directory;
    open_close[2] = binary_file_in_current_directory;
    open_close[3] = binary_file_in_path_directory;
    open_close[4] = binary_file_in_selected_directory;

    if( e->priority() ) {
        beginning = SEARCH_COUNT;
    } else {
        beginning = 1;
    }

    for(int i = beginning; i <= SEARCH_COUNT; ++i) {
        BINARY_TABLE = open_close[i]();

        if( BINARY_TABLE.length() > 0 ) {
            if( e->whichTable() ) {
                os << BINARY_TABLE << " is primarily accessible to Sjisutf8.";

                MessagesTexts m(DATA_TEXT, os.str(), PUT_NEW_LINE);
                m.setting();
            }

            break;
        } else {
            switch(i) {
            case 1: // beginning 
                if( e->noSearching() ) {
                    error_number = 291;
                    os << "setting(SearchTable *e) error-" << error_number << ": " << C_MS932_UTF16_BIN;

                    ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
                    x.setting();
                    exit(error_number);
                }

                break;
            case 4: // SEARCH_COUNT 
                error_number = 292;
                os << "setting(SearchTable *e) error-" << error_number << ": " << SEARCH_COUNT;

                x = new ErrorTexts(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
                x->setting();
                delete x;
                exit(error_number);
            default:
                continue;
            }

        }

    }

// Reaeding BINARY_TABLE to produce a table in the main memory 
    try {
        ifstream testFile( BINARY_TABLE.c_str() );
        testFile.exceptions(ios_base::failbit);
        r = true;
    } catch(...) {
        r = false;
        error_number = 293;
        os << "setting(SearchTable *e) error-" << error_number << ": " << BINARY_TABLE;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    clear();
    ifstream tableFile( BINARY_TABLE.c_str(), ios::binary );
    for(;;) {
        tableFile.read(a, E_SIZE);
        block_has_been_read = tableFile.gcount();
        if( block_has_been_read == 0 ) {
            break;
        }

        count += 1;
        if( count < FIRST_KANJI ) {
// ASCII 
            u = (unsigned char)a[0];
            k = (int)u;
        } else {
// kanji 
            u = (unsigned char)a[0];
            b = (int)u;
            k = b << 8;

            u = (unsigned char)a[1];
            b = (int)u;
            k += b;
        }

        populated(k, 0, a[2]);
        populated(k, 1, a[3]);
    }

// Producing another table in the main memory 
    for(int n = 0; n < U_SIZE; ++n) {
        UTF16_CHAR[n] = w;
        ++w;
    }

    return r;
}

bool SparsePopulationUTF16::setting(FinalEffects *f) {
    using namespace std;

    bool r;
    ErrorTexts *x;
    int b, beginning, block_has_been_read, error_number, k;
    int count = 0;
    string BINARY_TABLE;
    stringstream os;
    unsigned char u;
    wchar_t w = ZERO_CHAR;

// Finding MS932_UTF16_BIN 
    open_close[1] = binary_file_in_installation_directory;
    open_close[2] = binary_file_in_current_directory;
    open_close[3] = binary_file_in_path_directory;
    open_close[4] = binary_file_in_selected_directory;

    if( f->priority() ) {
        beginning = SEARCH_COUNT;
    } else {
        beginning = 1;
    }

    for(int i = beginning; i <= SEARCH_COUNT; ++i) {
        BINARY_TABLE = open_close[i]();

        if( BINARY_TABLE.length() > 0 ) {
            if( f->whichTable() ) {
                os << BINARY_TABLE << " is primarily accessible to Sjisutf8.";

                MessagesTexts m(DATA_TEXT, os.str(), PUT_NEW_LINE);
                m.setting();
            }

            break;
        } else {
            switch(i) {
            case 1: // beginning 
                if( f->noSearching() ) {
                    error_number = 294;
                    os << "setting(FinalEffects *f) error-" << error_number << ": " << C_MS932_UTF16_BIN;

                    ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
                    x.setting();
                    exit(error_number);
                }

                break;
            case 4: // SEARCH_COUNT 
                error_number = 295;
                os << "setting(FinalEffects *f) error-" << error_number << ": " << SEARCH_COUNT;

                x = new ErrorTexts(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
                x->setting();
                delete x;
                exit(error_number);
            default:
                continue;
            }

        }

    }

// Reaeding BINARY_TABLE to produce a table in the main memory 
    try {
        ifstream testFile( BINARY_TABLE.c_str() );
        testFile.exceptions(ios_base::failbit);
        r = true;
    } catch(...) {
        r = false;
        error_number = 296;
        os << "setting(FinalEffects *f) error-" << error_number << ": " << BINARY_TABLE;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    clear();
    ifstream tableFile( BINARY_TABLE.c_str(), ios::binary );
    for(;;) {
        tableFile.read(a, E_SIZE);
        block_has_been_read = tableFile.gcount();
        if( block_has_been_read == 0 ) {
            break;
        }

        count += 1;
        if( count < FIRST_KANJI ) {
// ASCII 
            u = (unsigned char)a[0];
            k = (int)u;
        } else {
// kanji 
            u = (unsigned char)a[0];
            b = (int)u;
            k = b << 8;

            u = (unsigned char)a[1];
            b = (int)u;
            k += b;
        }

        populated(k, 0, a[2]);
        populated(k, 1, a[3]);
    }

// Producing another table in the main memory 
    for(int n = 0; n < U_SIZE; ++n) {
        UTF16_CHAR[n] = w;
        ++w;
    }

    return r;
}

char SparsePopulationUTF16::working(int index, int place) {
    int k = UTF16_KANJI_SIZE_1 * index + place;
    return z[k];
}

wchar_t SparsePopulationUTF16::UTF16Char(int index) {
    return UTF16_CHAR[index];
}

void SparsePopulationUTF16::populated(int index, int place, char c) {
    int k = UTF16_KANJI_SIZE_1 * index + place;
    z[k] = c;
}

void SparsePopulationUTF16::clear() {
    for(int k = 0; k < Z_SIZE; ++k) {
        z[k] = 0;
    }

}

