#include "StandardIO.h"

namespace {
    char DATA_TEXT[] = "data.txt";
    char TEMPORARY_BINARY[] = "binary.tmp";
    char TEMPORARY_TEXT[] = "text.tmp";
    const bool PUT_NEW_LINE = true;
    const char H_0A_C = 10; // LF 
    const char H_0D_C = 13; // CR 
    const int BUFFER_ERROR = -1;
    const int COMBO_ERROR = -3;
    const int EXPANSION_FACTOR = 4;
    const int FILE_ERROR = -2;
    const int H_0A = 10; // LF 
    const int H_0D = 13; // CR 
    const int H_81 = 129;
    const int H_FC = 252;
    const int KANA_HANDAKUTEN = 223; // DFh 
    const int KANA_KUTEN = 161; // A1h 
    const int KANJI_SIZE = 3;
    const int KATAKANA_SIZE = 3;
    const int REMOVED = -5;
    const int ZERO_LENGTH_ERROR = -4;
    const std::string C_ERR = "err";
    const std::string COMMAND_NAME_FILE = "sjisutf8.txt";
    const std::string EMPTY = "";
    const std::string TEXT_FILE = ".txt";
    const unsigned char BELL = 7;
    const unsigned char COMMA_BYTE = 44; // 2C, "," 
}

StandardInput::~StandardInput() {
    delete[] buffer;
    delete[] combo;
    delete table;
}

StandardInput::StandardInput() {
    beginning = 0;
    BELL_ASCII_KATAKANA[0] = 7; BELL_ASCII_KATAKANA[1] = 7; BELL_ASCII_KATAKANA[2] = 7; BELL_ASCII_KATAKANA[3] = 7; BELL_ASCII_KATAKANA[4] = 7; BELL_ASCII_KATAKANA[5] = 7;
    BELL_ASCII_KATAKANA[6] = 7; BELL_ASCII_KATAKANA[7] = 7; BELL_ASCII_KATAKANA[8] = 7; BELL_ASCII_KATAKANA[9] = 7; BELL_ASCII_KATAKANA[10] = 7; BELL_ASCII_KATAKANA[11] = 7;
    BELL_ASCII_KATAKANA[12] = 7; BELL_ASCII_KATAKANA[13] = 7; BELL_ASCII_KATAKANA[14] = 7; BELL_ASCII_KATAKANA[15] = 7; BELL_ASCII_KATAKANA[16] = 7; BELL_ASCII_KATAKANA[17] = 7;
    BELL_ASCII_KATAKANA[18] = 7; BELL_ASCII_KATAKANA[19] = 7; BELL_ASCII_KATAKANA[20] = 7; BELL_ASCII_KATAKANA[21] = 7; BELL_ASCII_KATAKANA[22] = 7; BELL_ASCII_KATAKANA[23] = 7;
    BELL_ASCII_KATAKANA[24] = 7; BELL_ASCII_KATAKANA[25] = 7; BELL_ASCII_KATAKANA[26] = 7; BELL_ASCII_KATAKANA[27] = 7; BELL_ASCII_KATAKANA[28] = 7; BELL_ASCII_KATAKANA[29] = 7;
    BELL_ASCII_KATAKANA[30] = 7; BELL_ASCII_KATAKANA[31] = 7; BELL_ASCII_KATAKANA[32] = 32; BELL_ASCII_KATAKANA[33] = 33; BELL_ASCII_KATAKANA[34] = 34; BELL_ASCII_KATAKANA[35] = 35;
    BELL_ASCII_KATAKANA[36] = 36; BELL_ASCII_KATAKANA[37] = 37; BELL_ASCII_KATAKANA[38] = 38; BELL_ASCII_KATAKANA[39] = 39; BELL_ASCII_KATAKANA[40] = 40; BELL_ASCII_KATAKANA[41] = 41;
    BELL_ASCII_KATAKANA[42] = 42; BELL_ASCII_KATAKANA[43] = 43; BELL_ASCII_KATAKANA[44] = 44; BELL_ASCII_KATAKANA[45] = 45; BELL_ASCII_KATAKANA[46] = 46; BELL_ASCII_KATAKANA[47] = 47;
    BELL_ASCII_KATAKANA[48] = 48; BELL_ASCII_KATAKANA[49] = 49; BELL_ASCII_KATAKANA[50] = 50; BELL_ASCII_KATAKANA[51] = 51; BELL_ASCII_KATAKANA[52] = 52; BELL_ASCII_KATAKANA[53] = 53;
    BELL_ASCII_KATAKANA[54] = 54; BELL_ASCII_KATAKANA[55] = 55; BELL_ASCII_KATAKANA[56] = 56; BELL_ASCII_KATAKANA[57] = 57; BELL_ASCII_KATAKANA[58] = 58; BELL_ASCII_KATAKANA[59] = 59;
    BELL_ASCII_KATAKANA[60] = 60; BELL_ASCII_KATAKANA[61] = 61; BELL_ASCII_KATAKANA[62] = 62; BELL_ASCII_KATAKANA[63] = 63; BELL_ASCII_KATAKANA[64] = 64; BELL_ASCII_KATAKANA[65] = 65;
    BELL_ASCII_KATAKANA[66] = 66; BELL_ASCII_KATAKANA[67] = 67; BELL_ASCII_KATAKANA[68] = 68; BELL_ASCII_KATAKANA[69] = 69; BELL_ASCII_KATAKANA[70] = 70; BELL_ASCII_KATAKANA[71] = 71;
    BELL_ASCII_KATAKANA[72] = 72; BELL_ASCII_KATAKANA[73] = 73; BELL_ASCII_KATAKANA[74] = 74; BELL_ASCII_KATAKANA[75] = 75; BELL_ASCII_KATAKANA[76] = 76; BELL_ASCII_KATAKANA[77] = 77;
    BELL_ASCII_KATAKANA[78] = 78; BELL_ASCII_KATAKANA[79] = 79; BELL_ASCII_KATAKANA[80] = 80; BELL_ASCII_KATAKANA[81] = 81; BELL_ASCII_KATAKANA[82] = 82; BELL_ASCII_KATAKANA[83] = 83;
    BELL_ASCII_KATAKANA[84] = 84; BELL_ASCII_KATAKANA[85] = 85; BELL_ASCII_KATAKANA[86] = 86; BELL_ASCII_KATAKANA[87] = 87; BELL_ASCII_KATAKANA[88] = 88; BELL_ASCII_KATAKANA[89] = 89;
    BELL_ASCII_KATAKANA[90] = 90; BELL_ASCII_KATAKANA[91] = 91; BELL_ASCII_KATAKANA[92] = 92; BELL_ASCII_KATAKANA[93] = 93; BELL_ASCII_KATAKANA[94] = 94; BELL_ASCII_KATAKANA[95] = 95;
    BELL_ASCII_KATAKANA[96] = 96; BELL_ASCII_KATAKANA[97] = 97; BELL_ASCII_KATAKANA[98] = 98; BELL_ASCII_KATAKANA[99] = 99; BELL_ASCII_KATAKANA[100] = 100; BELL_ASCII_KATAKANA[101] = 101;
    BELL_ASCII_KATAKANA[102] = 102; BELL_ASCII_KATAKANA[103] = 103; BELL_ASCII_KATAKANA[104] = 104; BELL_ASCII_KATAKANA[105] = 105; BELL_ASCII_KATAKANA[106] = 106; BELL_ASCII_KATAKANA[107] = 107;
    BELL_ASCII_KATAKANA[108] = 108; BELL_ASCII_KATAKANA[109] = 109; BELL_ASCII_KATAKANA[110] = 110; BELL_ASCII_KATAKANA[111] = 111; BELL_ASCII_KATAKANA[112] = 112; BELL_ASCII_KATAKANA[113] = 113;
    BELL_ASCII_KATAKANA[114] = 114; BELL_ASCII_KATAKANA[115] = 115; BELL_ASCII_KATAKANA[116] = 116; BELL_ASCII_KATAKANA[117] = 117; BELL_ASCII_KATAKANA[118] = 118; BELL_ASCII_KATAKANA[119] = 119;
    BELL_ASCII_KATAKANA[120] = 120; BELL_ASCII_KATAKANA[121] = 121; BELL_ASCII_KATAKANA[122] = 122; BELL_ASCII_KATAKANA[123] = 123; BELL_ASCII_KATAKANA[124] = 124; BELL_ASCII_KATAKANA[125] = 125;
    BELL_ASCII_KATAKANA[126] = 126; BELL_ASCII_KATAKANA[127] = 7; BELL_ASCII_KATAKANA[128] = 7; BELL_ASCII_KATAKANA[129] = 7; BELL_ASCII_KATAKANA[130] = 7; BELL_ASCII_KATAKANA[131] = 7;
    BELL_ASCII_KATAKANA[132] = 7; BELL_ASCII_KATAKANA[133] = 7; BELL_ASCII_KATAKANA[134] = 7; BELL_ASCII_KATAKANA[135] = 7; BELL_ASCII_KATAKANA[136] = 7; BELL_ASCII_KATAKANA[137] = 7;
    BELL_ASCII_KATAKANA[138] = 7; BELL_ASCII_KATAKANA[139] = 7; BELL_ASCII_KATAKANA[140] = 7; BELL_ASCII_KATAKANA[141] = 7; BELL_ASCII_KATAKANA[142] = 7; BELL_ASCII_KATAKANA[143] = 7;
    BELL_ASCII_KATAKANA[144] = 7; BELL_ASCII_KATAKANA[145] = 7; BELL_ASCII_KATAKANA[146] = 7; BELL_ASCII_KATAKANA[147] = 7; BELL_ASCII_KATAKANA[148] = 7; BELL_ASCII_KATAKANA[149] = 7;
    BELL_ASCII_KATAKANA[150] = 7; BELL_ASCII_KATAKANA[151] = 7; BELL_ASCII_KATAKANA[152] = 7; BELL_ASCII_KATAKANA[153] = 7; BELL_ASCII_KATAKANA[154] = 7; BELL_ASCII_KATAKANA[155] = 7;
    BELL_ASCII_KATAKANA[156] = 7; BELL_ASCII_KATAKANA[157] = 7; BELL_ASCII_KATAKANA[158] = 7; BELL_ASCII_KATAKANA[159] = 7; BELL_ASCII_KATAKANA[160] = 7; BELL_ASCII_KATAKANA[161] = 161; // A1h 
    BELL_ASCII_KATAKANA[162] = 162; BELL_ASCII_KATAKANA[163] = 163; BELL_ASCII_KATAKANA[164] = 164; BELL_ASCII_KATAKANA[165] = 165; BELL_ASCII_KATAKANA[166] = 166; BELL_ASCII_KATAKANA[167] = 167;
    BELL_ASCII_KATAKANA[168] = 168; BELL_ASCII_KATAKANA[169] = 169; BELL_ASCII_KATAKANA[170] = 170; BELL_ASCII_KATAKANA[171] = 171; BELL_ASCII_KATAKANA[172] = 172; BELL_ASCII_KATAKANA[173] = 173;
    BELL_ASCII_KATAKANA[174] = 174; BELL_ASCII_KATAKANA[175] = 175; BELL_ASCII_KATAKANA[176] = 176; BELL_ASCII_KATAKANA[177] = 177; BELL_ASCII_KATAKANA[178] = 178; BELL_ASCII_KATAKANA[179] = 179;
    BELL_ASCII_KATAKANA[180] = 180; BELL_ASCII_KATAKANA[181] = 181; BELL_ASCII_KATAKANA[182] = 182; BELL_ASCII_KATAKANA[183] = 183; BELL_ASCII_KATAKANA[184] = 184; BELL_ASCII_KATAKANA[185] = 185;
    BELL_ASCII_KATAKANA[186] = 186; BELL_ASCII_KATAKANA[187] = 187; BELL_ASCII_KATAKANA[188] = 188; BELL_ASCII_KATAKANA[189] = 189; BELL_ASCII_KATAKANA[190] = 190; BELL_ASCII_KATAKANA[191] = 191;
    BELL_ASCII_KATAKANA[192] = 192; BELL_ASCII_KATAKANA[193] = 193; BELL_ASCII_KATAKANA[194] = 194; BELL_ASCII_KATAKANA[195] = 195; BELL_ASCII_KATAKANA[196] = 196; BELL_ASCII_KATAKANA[197] = 197;
    BELL_ASCII_KATAKANA[198] = 198; BELL_ASCII_KATAKANA[199] = 199; BELL_ASCII_KATAKANA[200] = 200; BELL_ASCII_KATAKANA[201] = 201; BELL_ASCII_KATAKANA[202] = 202; BELL_ASCII_KATAKANA[203] = 203;
    BELL_ASCII_KATAKANA[204] = 204; BELL_ASCII_KATAKANA[205] = 205; BELL_ASCII_KATAKANA[206] = 206; BELL_ASCII_KATAKANA[207] = 207; BELL_ASCII_KATAKANA[208] = 208; BELL_ASCII_KATAKANA[209] = 209;
    BELL_ASCII_KATAKANA[210] = 210; BELL_ASCII_KATAKANA[211] = 211; BELL_ASCII_KATAKANA[212] = 212; BELL_ASCII_KATAKANA[213] = 213; BELL_ASCII_KATAKANA[214] = 214; BELL_ASCII_KATAKANA[215] = 215;
    BELL_ASCII_KATAKANA[216] = 216; BELL_ASCII_KATAKANA[217] = 217; BELL_ASCII_KATAKANA[218] = 218; BELL_ASCII_KATAKANA[219] = 219; BELL_ASCII_KATAKANA[220] = 220; BELL_ASCII_KATAKANA[221] = 221;
    BELL_ASCII_KATAKANA[222] = 222; BELL_ASCII_KATAKANA[223] = 223;
}

bool StandardInput::displayDataText() {
    using namespace std;

    bool b;
    char c[C_SIZE];
    int error_number;
    stringstream os;

    try {
        ifstream testFile(DATA_TEXT, ios::binary);
        testFile.exceptions(ios_base::failbit);
        b = true;
    } catch(...) {
        b = false;
        error_number = 197;
        os << "displayDataText() error-" << error_number << ": " << DATA_TEXT;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    ifstream dataFile(DATA_TEXT);
    while( dataFile.getline(c, C_SIZE) ) {
        cout << c << endl;
    }

    return b;
}

int StandardInput::bufferingWriting(int comboLength) {
    using namespace std;

    int n = 0;
    ofstream newFile( new_name.c_str(), ios::binary );

    y_length = Y_SIZE;
    while( n < comboLength ) {
        for(int k = 0; k < Y_SIZE; ++k) {
            y[k] = combo[n++];
            if( n < comboLength ) {
                continue;
            } else {
                y_length = k + 1;
                break;
            }

        }

        newFile.write(y, y_length);
    }

    return n; // n == comboLength
}

int StandardInput::lineCount() const {
    return count;
}

int StandardInput::produceBinaryFile(std::string & name, bool no_removing) {
    using namespace std;

    int block_has_been_read, error_number;
    int sum = 0;
    ofstream newFile(TEMPORARY_BINARY, ios::binary);
    stringstream os;

    try {
        ifstream testFile( name.c_str() );
        testFile.exceptions(ios_base::failbit);
    } catch(...) {
        error_number = 198;
        os << "produceBinaryFile() error-" << error_number << ": " << name;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    ifstream nameFile( name.c_str(), ios::binary );

    for(;;) {
        nameFile.read(y, Y_SIZE);
        block_has_been_read = nameFile.gcount();
        if( block_has_been_read == 0 ) {
            break;
        }

        newFile.write(y, block_has_been_read);
        sum += block_has_been_read;
    }

    nameFile.close();
    if(! no_removing ) {
        remove( name.c_str() );
    }

    return sum;
}

int StandardInput::produceTemporaryTextFile() {
    using namespace std;

    const int NEW_LINE_LENGTH = 2;
    int sum = 0;
    ofstream newFile(TEMPORARY_TEXT);
    stringstream os;

    while( cin.getline(y, Y_SIZE) ) {
        newFile << y << endl;
        sum += strlen(y);
        sum += NEW_LINE_LENGTH;
    }

    return sum;
}

int StandardInput::renameBinaryFile(std::string & name, bool no_removing) {
    using namespace std;

    char c[C_SIZE];
    int block_has_been_read, error_number;
    int sum = 0;
    stringstream os;

    try {
        ifstream testFile(TEMPORARY_BINARY);
        testFile.exceptions(ios_base::failbit);
    } catch(...) {
        error_number = 199;
        os << "renameBinaryFile() error-" << error_number << ": " << name;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    if( no_removing ) {
        ifstream nameFile(TEMPORARY_BINARY, ios::binary);
        ofstream newFile( name.c_str(), ios::binary );

        for(;;) {
            nameFile.read(y, Y_SIZE);
            block_has_been_read = nameFile.gcount();
            if( block_has_been_read == 0 ) {
                break;
            }

            newFile.write(y, block_has_been_read);
            sum += block_has_been_read;
        }

    } else {
        strcpy(c, name.c_str() );
        if( rename(TEMPORARY_BINARY, c) ) {
            remove(c);
            sum = REMOVED;
            rename(TEMPORARY_BINARY, c);
        }

    }

    return sum;
}

int StandardInput::setBufferSize() {
    using namespace std;

    int r = buffer_size;
    int error_number;
    stringstream os;

    try {
        buffer = new char[ buffer_size + 1 ];
    } catch(...) {
        r = BUFFER_ERROR;
        error_number = 200;
        os << "setBufferSize() error-" << error_number << ": " << "buffer_size = " << buffer_size;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
    }

    try {
        combo = new char[ combo_size + 1 ];
    } catch(...) {
        r = COMBO_ERROR;
        error_number = 201;
        os << "setBufferSize() error-" << error_number << ": " << "combo_size = " << combo_size;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
    }

    return r;
}

int StandardInput::standard_input() {
    using namespace std;

    bool kanji = false;
    char c;
    int block_has_been_read, error_number, first_SJIS_byte, k;
    int m = 0;
    int SJIS_code;
    stringstream os;
    unsigned char u;

    carriage_return_appears = false;
    count = 0;

// Reading and copying the file to produce a large buffer
    try {
        ifstream testFile(TEMPORARY_TEXT);
        testFile.exceptions(ios_base::failbit);
    } catch(...) {
        error_number = 202;
        os << "standard_input() error-" << error_number << ": " << TEMPORARY_TEXT;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    ifstream nameFile(TEMPORARY_TEXT, ios::binary);
    while( m < buffer_size ) {
        nameFile.read(y, Y_SIZE);
        block_has_been_read = nameFile.gcount();
        if( block_has_been_read == 0 ) {
            break;
        }

        for(int n = 0; n < Y_SIZE; ++n) {
            buffer[m++] = y[n];
            if( m < buffer_size ) {
                continue;
            } else {
                break;
            }

        }

    }

// Finding the first NEW_LINE 
    first_SJIS_byte = COMMA_BYTE;
    m = 0;
    for(int n = beginning; n < buffer_size; ++n) {
        beginning = n + 1;

        c = buffer[n];
        u = (unsigned char)c;
        k = (int)u;

        if( kanji ) {
            SJIS_code = first_SJIS_byte + k;

            combo[m] = table->working(SJIS_code, 0);
            combo[ m + 1 ] = table->working(SJIS_code, 1);
            combo[ m + 2 ] = table->working(SJIS_code, 2);
            m += KANJI_SIZE;

            kanji = false;
        } else {
            if( k == H_0A ) {
                count = 1;

                combo[m++] = H_0A_C;
                break;
            } else if( k == H_0D ) {
                combo[m++] = H_0D_C;

                carriage_return_appears = true;
                NEW_LINE_SIZE = 2; // 0Dh and 0Ah 
                NEW_LINE = EMPTY + H_0D_C;
                NEW_LINE = NEW_LINE + H_0A_C;
            } else if( H_81 <= k && k <= H_FC ) {
                if( KANA_KUTEN <= k && k <= KANA_HANDAKUTEN ) {
                    combo[m] = table->working(k, 0);
                    combo[ m + 1 ] = table->working(k, 1);
                    combo[ m + 2 ] = table->working(k, 2);
                    m += KATAKANA_SIZE;
                    kanji = false;
                } else {
                    first_SJIS_byte = k << 8;
                    kanji = true;
                }

            } else {
// ASCII 
                combo[m++] = c;
            }

        } // if( kanji ) , else

    }

// Counting lines 
    first_SJIS_byte = COMMA_BYTE;
    kanji = false;
    for(int n = beginning; n < buffer_size; ++n) {
        c = buffer[n];
        u = (unsigned char)c;
        k = (int)u;

        if( kanji ) {
            SJIS_code = first_SJIS_byte + k;

            combo[m] = table->working(SJIS_code, 0);
            combo[ m + 1 ] = table->working(SJIS_code, 1);
            combo[ m + 2 ] = table->working(SJIS_code, 2);
            m += KANJI_SIZE;

            kanji = false;
        } else {
            switch(k) {
            case H_0A:
                count += 1;
                combo[m++] = H_0A_C;
                break;
            case H_0D:
                combo[m++] = H_0D_C;
                break;
            default:
                if( H_81 <= k && k <= H_FC ) {
                    if( KANA_KUTEN <= k && k <= KANA_HANDAKUTEN) {
                        combo[m] = table->working(k, 0);
                        combo[ m + 1 ] = table->working(k, 1);
                        combo[ m + 2 ] = table->working(k, 2);
                        m += KATAKANA_SIZE;
                        kanji = false;
                    } else {
                        first_SJIS_byte = k << 8;
                        kanji = true;
                    }

                } else {
// ASCII 
                    combo[m++] = c;
                }

                break;
            }

        } // if( kanji ) , else

    }

    return bufferingWriting(m);
}

int StandardInput::standard_input_of_non_controllers() {
    using namespace std;

    bool kanji = false;
    char c;
    int block_has_been_read, error_number, first_SJIS_byte, k;
    int m = 0;
    int SJIS_code;
    stringstream os;
    unsigned char b, u;

    carriage_return_appears = false;
    count = 0;

// Reading and copying the file to produce a large buffer
    try {
        ifstream testFile(TEMPORARY_TEXT);
        testFile.exceptions(ios_base::failbit);
    } catch(...) {
        error_number = 203;
        os << "standard_input_of_non_controllers() error-" << error_number << ": " << TEMPORARY_TEXT;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    ifstream nameFile(TEMPORARY_TEXT, ios::binary);
    while( m < buffer_size ) {
        nameFile.read(y, Y_SIZE);
        block_has_been_read = nameFile.gcount();
        if( block_has_been_read == 0 ) {
            break;
        }

        for(int n = 0; n < Y_SIZE; ++n) {
            buffer[m++] = y[n];
            if( m < buffer_size ) {
                continue;
            } else {
                break;
            }

        }

    }

// Finding the first NEW_LINE 
    first_SJIS_byte = COMMA_BYTE;
    m = 0;
    for(int n = beginning; n < buffer_size; ++n) {
        beginning = n + 1;

        c = buffer[n];
        u = (unsigned char)c;
        k = (int)u;

        if( kanji ) {
            SJIS_code = first_SJIS_byte + k;

            combo[m] = table->working(SJIS_code, 0);
            combo[ m + 1 ] = table->working(SJIS_code, 1);
            combo[ m + 2 ] = table->working(SJIS_code, 2);
            m += KANJI_SIZE;

            kanji = false;
        } else {
            if( k == H_0A ) {
                count = 1;

                combo[m++] = H_0A_C;
                break;
            } else if( k == H_0D ) {
                combo[m++] = H_0D_C;

                carriage_return_appears = true;
                NEW_LINE_SIZE = 2; // 0Dh and 0Ah 
                NEW_LINE = EMPTY + H_0D_C;
                NEW_LINE = NEW_LINE + H_0A_C;
            } else if( H_81 <= k && k <= H_FC ) {
                if( KANA_KUTEN <= k && k <= KANA_HANDAKUTEN ) {
                    combo[m] = table->working(k, 0);
                    combo[ m + 1 ] = table->working(k, 1);
                    combo[ m + 2 ] = table->working(k, 2);
                    m += KATAKANA_SIZE;
                    kanji = false;
                } else {
                    first_SJIS_byte = k << 8;
                    kanji = true;
                }

            } else {
// ASCII 
                b = BELL_ASCII_KATAKANA[k];
                if( b == BELL ) {
                    continue; // Removing control characters 
                } else {
                    combo[m++] = c;
                }

            }

        } // if( kanji ) , else

    }

// Counting lines 
    first_SJIS_byte = COMMA_BYTE;
    kanji = false;
    for(int n = beginning; n < buffer_size; ++n) {
        c = buffer[n];
        u = (unsigned char)c;
        k = (int)u;

        if( kanji ) {
            SJIS_code = first_SJIS_byte + k;

            combo[m] = table->working(SJIS_code, 0);
            combo[ m + 1 ] = table->working(SJIS_code, 1);
            combo[ m + 2 ] = table->working(SJIS_code, 2);
            m += KANJI_SIZE;

            kanji = false;
        } else {
            switch(k) {
            case H_0A:
                count += 1;
                combo[m++] = H_0A_C;
                break;
            case H_0D:
                combo[m++] = H_0D_C;
                break;
            default:
                if( H_81 <= k && k <= H_FC ) {
                    if( KANA_KUTEN <= k && k <= KANA_HANDAKUTEN) {
                        combo[m] = table->working(k, 0);
                        combo[ m + 1 ] = table->working(k, 1);
                        combo[ m + 2 ] = table->working(k, 2);
                        m += KATAKANA_SIZE;
                        kanji = false;
                    } else {
                        first_SJIS_byte = k << 8;
                        kanji = true;
                    }

                } else {
// ASCII 
                    b = BELL_ASCII_KATAKANA[k];
                    if( b == BELL ) {
                        continue; // Removing control characters 
                    } else {
                        combo[m++] = c;
                    }

                }

                break;
            }

        } // if( kanji ) , else

    }

    return bufferingWriting(m);
}

long StandardInput::file_size(std::string & name) {
    using namespace std;

    int error_number;
    long length;
    stringstream os;

    try {
        ifstream testFile( name.c_str(), ios::binary | ios::ate );
        testFile.exceptions(ios_base::failbit);

        length = (long)testFile.tellg();
    } catch(...) {
        length = FILE_ERROR;
        error_number = 204;
        os << "file_size() error-" << error_number << ": " << name;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    if( length == 0L ) {
        length = ZERO_LENGTH_ERROR;
        error_number = 205;
        os << "file_size() error-" << error_number << ": length = " << length;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    return length;
}

void StandardInput::setting(FinalEffects *final_effects) {
    using namespace std;

    string name(TEMPORARY_TEXT);

    new_name = final_effects->newName();

    buffer_size = (int)file_size(name);
    combo_size = EXPANSION_FACTOR * buffer_size;
    NEW_LINE = EMPTY + H_0A_C;
    NEW_LINE_SIZE = 1;

    setBufferSize();

// Producing a conversion table in the main memory 
    table = new SparsePopulationUTF8();
    table->setting(final_effects);
}

void StandardInput::setting(FinalEffects *final_effects, int b) {
    buffer_size = b;
    combo_size = EXPANSION_FACTOR * buffer_size;

    new_name = final_effects->newName();

    NEW_LINE = EMPTY + H_0A_C;
    NEW_LINE_SIZE = 1;

    setBufferSize();

// Producing a conversion table in the main memory 
    table = new SparsePopulationUTF8();
    table->setting(final_effects);
}

void StandardInput::setting(SearchTable *search_table) {
    std::string name(TEMPORARY_TEXT);

    new_name = COMMAND_NAME_FILE;

    buffer_size = (int)file_size(name);
    combo_size = EXPANSION_FACTOR * buffer_size;

    NEW_LINE = EMPTY + H_0A_C;
    NEW_LINE_SIZE = 1;

    setBufferSize();

// Producing a conversion table in the main memory 
    table = new SparsePopulationUTF8();
    table->setting(search_table);
}

