/*
utf8to16 C++ Version 1.02
Copyright 2020 Sogaya
Producing a UTF-16 text from the UTF-8 text

Borland C++ 5.5.1 for Win32 (BCC55)
Digital Mars Compiler Version 8.42n (DMC 8.57 [dm857c.zip] ), STLport 4.5.3
Embarcadero C++ 7.20 for Win32 (BCC101)
Embarcadero C++ 7.30 for Win32 (BCC102)
GCC 3.4.2 (mingw-special [Bloodshed Dev-C++ 4.9.9.2] )

*/

#include "ErrorText.h"
#include "FinalEffect.h"
#include "Guide.h"
#include "HyphenTerm.h"
#include "ProduceUTF16.h"
#include "ReadWrite.h"

namespace {
    char TEMPORARY_BINARY[] = "binary.tmp";
    bool REMOVE_ANCIENT_FILE = false;
    bool VERSION_WITH_COMMENTS = true;
    bool VERSION_WITHOUT_COMMENTS = false;
    char C_SETTING_TEXT[] = "C:\\Sjisutf8\\set_utf8to16.txt";
    char SETTING_TEXT[] = "set_utf8to16.txt";
    const char TAB_C = 9; // 09h
    const double VERSION_NUMBER = 1.02;
    const int BUFFER_ERROR = -1;
    std::string BYTE_ORDER_MARK = "-m";
    std::string COMMAND_NAME_FILE = "utf8to16.txt";
    std::string EMPTY = "";
    std::string INCLUDE_0DH = "-0";
    std::string PATH_NAME = "C:\\Sjisutf8\\sjisutf8.txt";
}

int main(int argc, char *argv[]) {
    using namespace std;

    bool SETTING_TEXT_IS_NOT_ACCESSIBLE;
    try {
        ifstream testFile(C_SETTING_TEXT);
        testFile.exceptions(ios_base::failbit);
        SETTING_TEXT_IS_NOT_ACCESSIBLE = false;
    } catch(...) {
        SETTING_TEXT_IS_NOT_ACCESSIBLE = true;
    }

    if( SETTING_TEXT_IS_NOT_ACCESSIBLE ) {
        try {
            ifstream testFile(SETTING_TEXT);
            testFile.exceptions(ios_base::failbit);
        } catch(...) {
// Providing SETTING_TEXT in the current directory
            ofstream newFile(SETTING_TEXT);

            newFile << PATH_NAME << endl;
            newFile << BYTE_ORDER_MARK << TAB_C << "BOM" << endl;
            newFile << INCLUDE_0DH << TAB_C << "CRLF" << endl;
        }

    }

    if( argc == 1 ) {
        Guiding *g = new Guiding();
        g->versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
        delete g;
        exit(242);
    }

// command parameters 
    bool carriage_return, remove_0Dh;
    int file_name_length, new_name_length;
    string file_name, new_name;
    ParameterList *listing = new ParameterList(argc, argv);
    HyphenTerms *h = new HyphenTerms(listing);
    h->setting();

    if( h->guideFile() ) {
        Guiding *g = new Guiding();
        g->versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
        delete g; delete h;
        exit(243);
    } else if( h->versionNumber() ) {
        Guiding *g = new Guiding();
        g->versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
        delete g; delete h;
        exit(244);
    }

    remove_0Dh = h->lineFeed();

    if( remove_0Dh ) {
        if( h->carriageReturn() ) {
            carriage_return = true; // Regardless of remove_0Dh, software recognizes that carriage_return has priority. 
        } else {
            carriage_return = false;
        }

    } else {
        carriage_return = true;
    }

    new_name = h->listName(1);
    new_name_length = new_name.length();
    if( new_name_length == 0 ) {
        new_name = h->newName();
    }

    h->joints(); // joint_byte 
    h->labels(); // label_byte
    h->quotes(); // quote_byte 
    h->separators(); // separator_byte 

    FinalEffects *f = new FinalEffects( h->fileName(), new_name, h->pathName(), 
        h->atPoint(), 
        h->jointByte(), 
        carriage_return, h->checkLength(), h->countLines(), h->identicalTime(), h->noRemoving(), remove_0Dh, h->produceTemporaryBinary(), h->withBOM() );

    file_name = f->fileName();
    file_name_length = file_name.length();
    if( file_name_length == 0 ) {
        Guiding *g = new Guiding();
        g->versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
        delete g; delete h;
        exit(245);
    }

    new_name = f->newName();
    new_name_length = new_name.length();
    if( new_name_length == 0 ) {
        Guiding *g = new Guiding();
        g->versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
        delete g; delete h;
        exit(246);
    }

    if( f->produceTemporaryBinary() ) {
        ReadingWriting *r = new ReadingWriting(f, TEMPORARY_BINARY);
        if( r->setting() ) {
            if( f->carriageReturn() ) {
                r->reading_to_produce_UTF16_CRLF();
            } else {
                r->reading_to_produce_UTF16_LF();
            }

            r->renameBinaryFile( new_name, f->noRemoving() );
        } else {
            exit(247);
        }

        delete r; delete f; delete h;
        return 0;
    }

    if( f->checkLength() ) {
        ReadingWriting *r = new ReadingWriting(f, TEMPORARY_BINARY);
        if( r->setting() ) {
            if( f->carriageReturn() ) {
                r->checking_length_to_produce_UTF16_CRLF();
            } else {
                r->checking_length_to_produce_UTF16_LF();
            }

            r->renameBinaryFile( new_name, f->noRemoving() );
        } else {
            exit(248);
        }

        delete r; delete f; delete h;
        return 0;
    }

    if( f->countLines() ) {
        ReadingWriting *r = new ReadingWriting(f, TEMPORARY_BINARY);
        if( r->setting() ) {
            int count = r->countLines(file_name);

            if( count == 1 ) {
                cout << count << " line" << endl;
            } else {
                cout << count << " lines" << endl;
            }

        } else {
            exit(249);
        }

        delete r; delete f; delete h;
        return 0;
    }

    CodePoint *c = new CodePoint( file_name, f->withBOM() );
    c->setting();
    c->ifErrorOccurs(BUFFER_ERROR);

    if( f->identicalTime() ) {
        c->produceBinaryFile(file_name, REMOVE_ANCIENT_FILE);
        c->renameBinaryFile( file_name, f->noRemoving() );
    }

    if( f->carriageReturn() ) {
        c->reading_to_produce_UTF16_CRLF(new_name);
    } else {
        if( f->remove0Dh() ) {
            c->reading_to_produce_UTF16_LF(new_name);
        } else {
            c->reading_to_produce_UTF16_text(new_name);
        }

    }

    if( SETTING_TEXT_IS_NOT_ACCESSIBLE ) {
        if(! f->noRemoving() ) {
            remove(SETTING_TEXT);
        }

    }

    delete f; delete h; delete listing;
    return 0;
}
